<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="webenginepart" version="12" translationDomain="webenginepart">
<MenuBar>
 <Menu name="file">
  <text>&amp;File</text>
  <Action name="saveDocument" />
  <Action name="saveFullHtmlPage"/>
  <Separator />
  <Action name="printPreview" group="print" />
 </Menu>
 <Menu name="edit">
  <text>&amp;Edit</text>
  <Action name="selectAll" />
  <Separator />
  <Action name="find" />
 </Menu>
 <Menu name="view">
  <text>&amp;View</text>
  <Action name="zoomIn" />
  <Action name="zoomOut" />
  <Action name="zoomNormal" />
  <Action name="zoomTextOnly" />
  <Action name="zoomToDPI" />
  <Separator />
  <Action name="setEncoding" />
  <Action name="viewDocumentSource" />
  <ActionList name="debugScriptList" />
 </Menu>
 <Menu name="tools">
   <text>&amp;Tools</text>
   <Menu name="wallet" icon="wallet-open">
     <text>&amp;Credential management</text>
     <Action name="walletFillFormsNow" enabled="false"/>
     <Action name="walletCacheFormsNow"/>
     <Separator />
     <Action name="walletRescan" />
     <Separator />
     <Action name="walletCustomizeFields"/>
     <Action name="walletRemoveCustomization"/>
     <Separator />
     <Action name="walletDisablePasswordCaching"/>
     <Action name="walletRemoveCachedData"/>
     <Separator />
     <Action name="walletShowManager"/>
     <Action name="walletCloseWallet"/>
  </Menu>
 </Menu>
</MenuBar>
<ToolBar name="htmlToolBar" iconText="icononly" iconSize="22" hidden="true"><text>HTML Toolbar</text>
  <Action name="zoomIn" />
  <Action name="zoomOut" />
  <Action name="zoomNormal" />
</ToolBar>
</gui>
