/* This file is part of the KDE project
    SPDX-FileCopyrightText: 1998, 1999 Torben Weis <weis@kde.org>
    SPDX-FileCopyrightText: 1999 David Faure <faure@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

// browser window color defaults -- Bernd
#define HTML_DEFAULT_LNK_COLOR Qt::blue
#define HTML_DEFAULT_TXT_COLOR Qt::black
#define HTML_DEFAULT_VLNK_COLOR Qt::magenta
#define HTML_DEFAULT_BASE_COLOR Qt::white

#define HTML_DEFAULT_VIEW_FONT "Sans Serif"
#define HTML_DEFAULT_VIEW_FIXED_FONT "Monospace"
#define HTML_DEFAULT_VIEW_SERIF_FONT "Serif"
#define HTML_DEFAULT_VIEW_SANSSERIF_FONT "Sans Serif"
#define HTML_DEFAULT_VIEW_CURSIVE_FONT "Cursive"
#define HTML_DEFAULT_VIEW_FANTASY_FONT "Fantasy"
#define HTML_DEFAULT_MIN_FONT_SIZE 7 // everything smaller is usually unreadable.
