<?xml version="1.0"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="Konqueror" version="74">
<MenuBar>
 <Menu name="file" noMerge="1"><text>&amp;File</text>
  <Action name="new_window"/>
  <Action name="newtab"/>
  <Separator/>
  <Action name="open_location"/>
  <Action name="open_file"/>
  <Action name="sessions"/>
  <Separator/>
  <Action name="sendURL"/>
  <Action name="sendPage"/>
  <Separator/>
  <Merge/> <!--Includes termination separator-->
  <Action name="print"/>
  <DefineGroup name="print" />
  <Separator/>
  <ActionList name="openwithbase"/>
  <ActionList name="openwith" /> <!--Includes termination separator-->
  <Action name="quit"/>
 </Menu>
 <Menu name="edit" noMerge="1"><text>&amp;Edit</text>  <!--Here is what applies to a selection-->
  <Action name="undo" />
  <Action name="cut"/>
  <Action name="copy"/>
  <Action name="paste"/>
  <!--<ActionList name="operations" />--><!-- copy files and move files, when two directory views -->
  <WeakSeparator/>
  <Merge/>
 </Menu>
 <Menu name="view" noMerge="1"><text>&amp;View</text> <!--Here is what applies to the active view-->
  <ActionList name="viewmode" />
  <Action name="usehtml"/>
  <Action name="lock"/>
  <Action name="link"/>
  <Separator/>
  <Action name="reload"/>
  <Action name="stop" />
  <WeakSeparator/>
  <Merge/>
 </Menu>
 <Menu name="go"><text>&amp;Go</text>
  <!-- go_up, go_back, go_forward, go_home: coming from ui_standards.rc -->
  <Merge/>
  <Action name="go_history"/>
  <Separator/>
  <Action name="go_most_often"/>
  <Action name="history"/>
  <Action name="closeditems"/>
 </Menu>
 <Action name="bookmarks"/>
 <Menu name="tools"><text>&amp;Tools</text>
   <Merge name="tools" />
   <Action name="inspect_page"/>
   <Separator />
 </Menu>
 <Menu name="settings" noMerge="1"><text>&amp;Settings</text>
  <Merge name="StandardToolBarMenuHandler" />
  <Action name="options_show_menubar"/>
  <Action name="options_show_statusbar"/>
  <ActionList name="toggleview" />
  <Separator/>
  <Action name="fullscreen"/>
  <Separator/>
  <Action name="loadviewprofile"/>
  <Action name="saveviewprofile"/>
  <Action name="saveremoveviewprofile"/>
  <Separator/>
  <Action name="switch_application_language"/>
  <Action name="options_configure_extensions"/>
  <Action name="configurespellcheck"/>
  <Action name="options_configure_keybinding"/>
  <Action name="options_configure_toolbars"/>
  <Action name="options_configure"/>
 </Menu>
 <Menu name="window" append="settings_merge"><text>&amp;Window</text>
  <Action name="splitviewh"/>
  <Action name="splitviewv"/>
  <Action name="removeview"/>
  <Separator/>
  <Action name="new_window"/>
  <Action name="duplicate_window"/>
  <Separator/>
  <Action name="newtab"/>
  <Action name="duplicatecurrenttab"/>
  <Action name="breakoffcurrenttab"/>
  <Action name="removecurrenttab"/>
  <Separator/>
  <Action name="tab_move_left"/>
  <Action name="tab_move_right"/>
  <Separator/>
  <Action name="protect_window"/>
 </Menu>
 <Menu name="help" append="about_merge"><text>&amp;Help</text>
  <Action name="konqintro"/>
 </Menu>
 <Merge/>
</MenuBar>
<ToolBar iconText="icononly" name="mainToolBar" newline="true" noMerge="1"><text>Main Toolbar</text>
 <Action name="go_back"/>
 <Action name="go_forward"/>
 <Action name="go_up"/>
 <Action name="reload"/>
 <Action name="stop"/>
 <Action name="go_home_popup"/>
 <ActionList name="fullscreen"/>
 <WeakSeparator/>
 <Merge/>
</ToolBar>
<ToolBar hidden="true" name="extraToolBar" newline="false" index="1"><text>Extra Toolbar</text>
  <Action name="konq_sidebartng" />
  <Action name="findfile" />
  <Action name="splitviewv" />
  <Action name="splitviewh" />
  <Action name="removeview" />
</ToolBar>
<ToolBar fullWidth="true" iconText="icononly" name="locationToolBar" newline="false"><text>Location Toolbar</text>
  <Action name="toolbar_url_combo" />
  <Action name="go_url" />
  <Merge />
  <Action name="konq_logo" />
</ToolBar>
<ToolBar noEdit="true" iconText="icontextright" fullWidth="true" name="bookmarkToolBar" iconSize="16" newline="true"><text>Bookmark Toolbar</text>
</ToolBar>
<MainWindow>
  <Action name="clear_location" />
</MainWindow>
<ActionProperties>
  <Action shortcut="F9" name="konq_sidebartng" />
</ActionProperties>

</gui>

